@@ -113,24 +113,20 @@ class Inventory(ModelWorkflow, ModelSQL, ModelView):
                         'inventory': new_id,
                         'move': False,
                         }, context=context)
-            self.complete_lines(cursor, user, new_id,
-                    product_ids=[x.product.id for x in inventory.lines],
-                    context=context)
+            self.complete_lines(cursor, user, new_id, context=context)
             new_ids.append(new_id)
 
         if int_id:
             return new_ids[0]
         return new_ids
 
-    def complete_lines(self, cursor, user, ids, product_ids=None, context=None):
+    def complete_lines(self, cursor, user, ids, context=None):
         '''
         Complete or update the inventories
 
         :param cursor: the database cursor
         :param user: the user id
         :param ids: the ids of stock.inventory
-        :param product_ids: the ids of product.product
-                if None all products are used
         :param context: the context
         '''
         line_obj = self.pool.get('stock.inventory.line')
@@ -149,8 +145,7 @@ class Inventory(ModelWorkflow, ModelSQL, ModelView):
             ctx = context and context.copy() or {}
             ctx['stock_date_end'] = inventory.date
             pbl = product_obj.products_by_location(
-                cursor, user, [inventory.location.id],
-                product_ids=product_ids, context=ctx)
+                cursor, user, [inventory.location.id], context=ctx)
 
             # Index some data
             product2uom = {}
@@ -166,41 +161,31 @@ class Inventory(ModelWorkflow, ModelSQL, ModelView):
 
             # Update existing lines
             for line in inventory.lines:
+                if not (line.product.active and
+                        line.product.type == 'stockable'):
+                    line_obj.delete(cursor, user, line.id, context=context)
+                    continue
                 if line.product.id in product_qty:
-                    quantity, uom_id = product_qty[line.product.id]
-                    del product_qty[line.product.id]
-                    # if nothing as changed, continue
-                    if line.quantity == line.expected_quantity == quantity \
-                            and line.uom.id == uom_id:
-                        continue
-                    values = {'expected_quantity': quantity,
-                              'uom': uom_id}
-                    # update also quantity field if not edited
-                    if line.quantity == line.expected_quantity:
-                        values['quantity'] = max(quantity, 0.0)
+                    quantity, uom_id = product_qty.pop(line.product.id)
+                elif line.product.id in product2uom:
+                    quantity, uom_id = 0.0, product2uom[line.product.id]
                 else:
-                    values = {'expected_quantity': 0.0,}
-                    if line.quantity == line.expected_quantity:
-                        values['quantity'] = 0
-
-
-                line_obj.write(
-                    cursor, user, line.id, values, context=context)
+                    quantity, uom_id = 0.0, line.product.default_uom.id
+                values = line_obj.update_values4complete(cursor, user,
+                        line, quantity, uom_id, context=context)
+                if values:
+                    line_obj.write(cursor, user, line.id, values,
+                            context=context)
 
             # Create lines if needed
-            for product in product_qty:
-                if product2type[product] != 'stockable':
+            for product_id in product_qty:
+                if product2type[product_id] != 'stockable':
                     continue
-                quantity, uom_id = product_qty[product]
-                values = {
-                    'product': product,
-                    'expected_quantity': quantity,
-                    'quantity': max(quantity, 0.0),
-                    'uom': uom_id,
-                    'inventory': inventory.id,
-                    }
-                line_obj.create(
-                    cursor, user, values, context=context)
+                quantity, uom_id = product_qty[product_id]
+                values = line_obj.create_values4complete(cursor, user,
+                        product_id, inventory, quantity, uom_id,
+                        context=context)
+                line_obj.create(cursor, user, values, context=context)
 
 Inventory()
 
@@ -274,6 +259,15 @@ class InventoryLine(ModelSQL, ModelView):
             context=context)
 
     def create_move(self, cursor, user, line, context=None):
+        '''
+        Create move for an inventory line
+
+        :param cursor: the database cursor
+        :param user: the user id
+        :param line: a BrowseRecord of inventory.line
+        :param context: the context
+        :return: the stock.move id or None
+        '''
         move_obj = self.pool.get('stock.move')
         uom_obj = self.pool.get('product.uom')
 
@@ -299,38 +293,63 @@ class InventoryLine(ModelSQL, ModelView):
             'effective_date': line.inventory.date,
             }, context=context)
         self.write(cursor, user, line.id, {'move': move_id}, context=context)
+        return move_id
+
+    def update_values4complete(self, cursor, user, line, quantity, uom_id,
+            context=None):
+        '''
+        Return update values to complete inventory
+
+        :param cursor: the database cursor
+        :param user: the user id
+        :param line: a BrowseRecord of inventory.line
+        :param quantity: the actual product quantity for the inventory location
+        :param uom_id: the UoM id of the product line
+        :param context: the context
+        :return: a dictionary
+        '''
+        res = {}
+        # if nothing changed, no update
+        if line.quantity == line.expected_quantity == quantity \
+                and line.uom.id == uom_id:
+            return {}
+        res['expected_quantity'] = quantity
+        res['uom'] = uom_id
+        # update also quantity field if not edited
+        if line.quantity == line.expected_quantity:
+            res['quantity'] = max(quantity, 0.0)
+        return res
+
+    def create_values4complete(self, cursor, user, product_id, inventory,
+            quantity, uom_id, context=None):
+        '''
+        Return create values to complete inventory
+
+        :param cursor: the database cursor
+        :param user: the user id
+        :param product_id: the product.product id
+        :param inventory: a BrowseRecord of inventory.inventory
+        :param quantity: the actual product quantity for the inventory location
+        :param uom_id: the UoM id of the product_id
+        :param context: the context
+        :return: a dictionary
+        '''
+        return {
+            'inventory': inventory.id,
+            'product': product_id,
+            'expected_quantity': quantity,
+            'quantity': max(quantity, 0.0),
+            'uom': uom_id,
+        }
 
 InventoryLine()
 
 
-class CompleteInventoryInit(ModelView):
-    'Complete Inventory Init'
-    _name = 'stock.inventory.complete.init'
-    _description = __doc__
-
-    products = fields.Many2Many('product.product', None, None,
-            'Products', domain=[('type', '=', 'stockable')])
-    categories = fields.Many2Many('product.category', None, None,
-            'Categories')
-
-CompleteInventoryInit()
-
-
 class CompleteInventory(Wizard):
     'Complete Inventory'
     _name = 'stock.inventory.complete'
     states = {
         'init': {
-            'result': {
-                'type': 'form',
-                'object': 'stock.inventory.complete.init',
-                'state': [
-                    ('end', 'Cancel', 'tryton-cancel'),
-                    ('complete', 'Complete', 'tryton-ok', True),
-                ],
-            },
-        },
-        'complete': {
             'result': {
                 'type': 'action',
                 'action': '_complete',
@@ -340,25 +359,8 @@ class CompleteInventory(Wizard):
         }
 
     def _complete(self, cursor, user, data, context=None):
-        category_obj = self.pool.get('product.category')
-        product_obj = self.pool.get('product.product')
         inventory_obj = self.pool.get('stock.inventory')
-
-        product_ids = data['form']['products'][0][1] or []
-        category_ids = data['form']['categories'][0][1] or []
-
-        if category_ids:
-            child_category_ids = category_obj.search(cursor, user,
-                    [('parent', 'child_of', category_ids)], context=context)
-            cat_product_ids = product_obj.search(cursor, user, [
-                ('category', 'in', child_category_ids),
-                ('type', '=', 'stockable'),
-                ], context=context)
-            if cat_product_ids:
-                product_ids += cat_product_ids
-
-        inventory_obj.complete_lines(cursor, user, data['ids'],
-                product_ids=product_ids, context=context)
+        inventory_obj.complete_lines(cursor, user, data['ids'], context=context)
 
         return {}
 
